@@ -8,20 +8,27 @@
 
 #include "AccessibleCaretLogger.h"
 #include "AccessibleCaretManager.h"
-#include "Layers.h"
-#include "gfxPrefs.h"
+
 #include "mozilla/AutoRestore.h"
-#include "mozilla/MouseEvents.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_layout.h"
+#include "mozilla/StaticPrefs_ui.h"
 #include "mozilla/TextEvents.h"
 #include "mozilla/TouchEvents.h"
-#include "mozilla/Preferences.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/MouseEventBinding.h"
+#include "mozilla/dom/Selection.h"
 #include "nsCanvasFrame.h"
 #include "nsDocShell.h"
 #include "nsFocusManager.h"
 #include "nsFrameSelection.h"
 #include "nsITimer.h"
+#include "nsLayoutUtils.h"
 #include "nsPresContext.h"
 
+using namespace mozilla;
+using namespace mozilla::dom;
+
 namespace mozilla {
 
 #undef AC_LOG
@@ -33,7 +40,7 @@ namespace mozilla {
   AC_LOGV_BASE("AccessibleCaretEventHub (%p): " message, this, ##__VA_ARGS__);
 
 NS_IMPL_ISUPPORTS(AccessibleCaretEventHub, nsIReflowObserver, nsIScrollObserver,
-                  nsISelectionListener, nsISupportsWeakReference);
+                  nsISupportsWeakReference);
 
 // -----------------------------------------------------------------------------
 // NoActionState
@@ -43,16 +50,17 @@ class AccessibleCaretEventHub::NoActionState
  public:
   const char* Name() const override { return "NoActionState"; }
 
+  MOZ_CAN_RUN_SCRIPT
   nsEventStatus OnPress(AccessibleCaretEventHub* aContext,
                         const nsPoint& aPoint, int32_t aTouchId,
                         EventClassID aEventClass) override {
     nsEventStatus rv = nsEventStatus_eIgnore;
 
     if (NS_SUCCEEDED(aContext->mManager->PressCaret(aPoint, aEventClass))) {
-      aContext->SetState(aContext->PressCaretState());
+      aContext->SetState(AccessibleCaretEventHub::PressCaretState());
       rv = nsEventStatus_eConsumeNoDefault;
     } else {
-      aContext->SetState(aContext->PressNoCaretState());
+      aContext->SetState(AccessibleCaretEventHub::PressNoCaretState());
     }
 
     aContext->mPressPoint = aPoint;
@@ -61,26 +69,30 @@ class AccessibleCaretEventHub::NoActionState
     return rv;
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnScrollStart(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnScrollStart();
-    aContext->SetState(aContext->ScrollState());
+    aContext->SetState(AccessibleCaretEventHub::ScrollState());
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnScrollPositionChanged(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnScrollPositionChanged();
   }
 
-  void OnSelectionChanged(AccessibleCaretEventHub* aContext,
-                          nsIDOMDocument* aDoc, nsISelection* aSel,
-                          int16_t aReason) override {
+  MOZ_CAN_RUN_SCRIPT
+  void OnSelectionChanged(AccessibleCaretEventHub* aContext, Document* aDoc,
+                          dom::Selection* aSel, int16_t aReason) override {
     aContext->mManager->OnSelectionChanged(aDoc, aSel, aReason);
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnBlur(AccessibleCaretEventHub* aContext,
               bool aIsLeavingDocument) override {
     aContext->mManager->OnBlur();
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnReflow(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnReflow();
   }
@@ -92,28 +104,33 @@ class AccessibleCaretEventHub::NoActionState
 };
 
 // -----------------------------------------------------------------------------
-// PressCaretState: Always consume the event since we've pressed on the caret.
+// PressCaretState: Because we've pressed on the caret, always consume the
+// event, both real and synthesized, so that other event handling code won't
+// have a chance to do something else to interrupt caret dragging.
 //
 class AccessibleCaretEventHub::PressCaretState
     : public AccessibleCaretEventHub::State {
  public:
   const char* Name() const override { return "PressCaretState"; }
 
-  nsEventStatus OnMove(AccessibleCaretEventHub* aContext,
-                       const nsPoint& aPoint) override {
-    if (aContext->MoveDistanceIsLarge(aPoint)) {
+  MOZ_CAN_RUN_SCRIPT
+  nsEventStatus OnMove(AccessibleCaretEventHub* aContext, const nsPoint& aPoint,
+                       WidgetMouseEvent::Reason aReason) override {
+    if (aReason == WidgetMouseEvent::eReal &&
+        aContext->MoveDistanceIsLarge(aPoint)) {
       if (NS_SUCCEEDED(aContext->mManager->DragCaret(aPoint))) {
-        aContext->SetState(aContext->DragCaretState());
+        aContext->SetState(AccessibleCaretEventHub::DragCaretState());
       }
     }
 
     return nsEventStatus_eConsumeNoDefault;
   }
 
+  MOZ_CAN_RUN_SCRIPT
   nsEventStatus OnRelease(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->ReleaseCaret();
     aContext->mManager->TapCaret(aContext->mPressPoint);
-    aContext->SetState(aContext->NoActionState());
+    aContext->SetState(AccessibleCaretEventHub::NoActionState());
 
     return nsEventStatus_eConsumeNoDefault;
   }
@@ -125,23 +142,29 @@ class AccessibleCaretEventHub::PressCaretState
 };
 
 // -----------------------------------------------------------------------------
-// DragCaretState: Always consume the event since we've pressed on the caret.
+// DragCaretState: Because we've pressed on the caret, always consume the event,
+// both real and synthesized, so that other event handling code won't have a
+// chance to do something else to interrupt caret dragging.
 //
 class AccessibleCaretEventHub::DragCaretState
     : public AccessibleCaretEventHub::State {
  public:
   const char* Name() const override { return "DragCaretState"; }
 
-  nsEventStatus OnMove(AccessibleCaretEventHub* aContext,
-                       const nsPoint& aPoint) override {
-    aContext->mManager->DragCaret(aPoint);
+  MOZ_CAN_RUN_SCRIPT
+  nsEventStatus OnMove(AccessibleCaretEventHub* aContext, const nsPoint& aPoint,
+                       WidgetMouseEvent::Reason aReason) override {
+    if (aReason == WidgetMouseEvent::eReal) {
+      aContext->mManager->DragCaret(aPoint);
+    }
 
     return nsEventStatus_eConsumeNoDefault;
   }
 
+  MOZ_CAN_RUN_SCRIPT
   nsEventStatus OnRelease(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->ReleaseCaret();
-    aContext->SetState(aContext->NoActionState());
+    aContext->SetState(AccessibleCaretEventHub::NoActionState());
 
     return nsEventStatus_eConsumeNoDefault;
   }
@@ -155,47 +178,51 @@ class AccessibleCaretEventHub::PressNoCaretState
  public:
   const char* Name() const override { return "PressNoCaretState"; }
 
-  nsEventStatus OnMove(AccessibleCaretEventHub* aContext,
-                       const nsPoint& aPoint) override {
+  nsEventStatus OnMove(AccessibleCaretEventHub* aContext, const nsPoint& aPoint,
+                       WidgetMouseEvent::Reason aReason) override {
     if (aContext->MoveDistanceIsLarge(aPoint)) {
-      aContext->SetState(aContext->NoActionState());
+      aContext->SetState(AccessibleCaretEventHub::NoActionState());
     }
 
     return nsEventStatus_eIgnore;
   }
 
   nsEventStatus OnRelease(AccessibleCaretEventHub* aContext) override {
-    aContext->SetState(aContext->NoActionState());
+    aContext->SetState(AccessibleCaretEventHub::NoActionState());
 
     return nsEventStatus_eIgnore;
   }
 
+  MOZ_CAN_RUN_SCRIPT
   nsEventStatus OnLongTap(AccessibleCaretEventHub* aContext,
                           const nsPoint& aPoint) override {
-    aContext->SetState(aContext->LongTapState());
+    aContext->SetState(AccessibleCaretEventHub::LongTapState());
 
     return aContext->GetState()->OnLongTap(aContext, aPoint);
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnScrollStart(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnScrollStart();
-    aContext->SetState(aContext->ScrollState());
+    aContext->SetState(AccessibleCaretEventHub::ScrollState());
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnBlur(AccessibleCaretEventHub* aContext,
               bool aIsLeavingDocument) override {
     aContext->mManager->OnBlur();
     if (aIsLeavingDocument) {
-      aContext->SetState(aContext->NoActionState());
+      aContext->SetState(AccessibleCaretEventHub::NoActionState());
     }
   }
 
-  void OnSelectionChanged(AccessibleCaretEventHub* aContext,
-                          nsIDOMDocument* aDoc, nsISelection* aSel,
-                          int16_t aReason) override {
+  MOZ_CAN_RUN_SCRIPT
+  void OnSelectionChanged(AccessibleCaretEventHub* aContext, Document* aDoc,
+                          dom::Selection* aSel, int16_t aReason) override {
     aContext->mManager->OnSelectionChanged(aDoc, aSel, aReason);
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnReflow(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnReflow();
   }
@@ -217,20 +244,23 @@ class AccessibleCaretEventHub::ScrollState
  public:
   const char* Name() const override { return "ScrollState"; }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnScrollEnd(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnScrollEnd();
-    aContext->SetState(aContext->NoActionState());
+    aContext->SetState(AccessibleCaretEventHub::NoActionState());
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnScrollPositionChanged(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnScrollPositionChanged();
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnBlur(AccessibleCaretEventHub* aContext,
               bool aIsLeavingDocument) override {
     aContext->mManager->OnBlur();
     if (aIsLeavingDocument) {
-      aContext->SetState(aContext->NoActionState());
+      aContext->SetState(AccessibleCaretEventHub::NoActionState());
     }
   }
 };
@@ -243,6 +273,7 @@ class AccessibleCaretEventHub::LongTapState
  public:
   const char* Name() const override { return "LongTapState"; }
 
+  MOZ_CAN_RUN_SCRIPT
   nsEventStatus OnLongTap(AccessibleCaretEventHub* aContext,
                           const nsPoint& aPoint) override {
     // In general text selection is lower-priority than the context menu. If
@@ -254,18 +285,20 @@ class AccessibleCaretEventHub::LongTapState
   }
 
   nsEventStatus OnRelease(AccessibleCaretEventHub* aContext) override {
-    aContext->SetState(aContext->NoActionState());
+    aContext->SetState(AccessibleCaretEventHub::NoActionState());
 
     // Do not consume the release since the press is not consumed in
     // PressNoCaretState either.
     return nsEventStatus_eIgnore;
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnScrollStart(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnScrollStart();
-    aContext->SetState(aContext->ScrollState());
+    aContext->SetState(AccessibleCaretEventHub::ScrollState());
   }
 
+  MOZ_CAN_RUN_SCRIPT
   void OnReflow(AccessibleCaretEventHub* aContext) override {
     aContext->mManager->OnReflow();
   }
@@ -295,25 +328,11 @@ MOZ_IMPL_STATE_CLASS_GETTER(PressNoCaretState)
 MOZ_IMPL_STATE_CLASS_GETTER(ScrollState)
 MOZ_IMPL_STATE_CLASS_GETTER(LongTapState)
 
-bool AccessibleCaretEventHub::sUseLongTapInjector = false;
-
-AccessibleCaretEventHub::AccessibleCaretEventHub(nsIPresShell* aPresShell)
-    : mPresShell(aPresShell) {
-  static bool prefsAdded = false;
-  if (!prefsAdded) {
-    Preferences::AddBoolVarCache(
-        &sUseLongTapInjector, "layout.accessiblecaret.use_long_tap_injector");
-    prefsAdded = true;
-  }
-}
+AccessibleCaretEventHub::AccessibleCaretEventHub(PresShell* aPresShell)
+    : mPresShell(aPresShell) {}
 
 void AccessibleCaretEventHub::Init() {
-  if (mInitialized && mManager) {
-    mManager->OnFrameReconstruction();
-  }
-
-  if (mInitialized || !mPresShell || !mPresShell->GetCanvasFrame() ||
-      !mPresShell->GetCanvasFrame()->GetCustomContentContainer()) {
+  if (mInitialized || !mPresShell) {
     return;
   }
 
@@ -329,7 +348,7 @@ void AccessibleCaretEventHub::Init() {
   nsPresContext* presContext = mPresShell->GetPresContext();
   MOZ_ASSERT(presContext, "PresContext should be given in PresShell::Init()");
 
-  nsIDocShell* docShell = presContext->GetDocShell();
+  nsDocShell* docShell = presContext->GetDocShell();
   if (!docShell) {
     return;
   }
@@ -339,7 +358,7 @@ void AccessibleCaretEventHub::Init() {
 
   mDocShell = static_cast<nsDocShell*>(docShell);
 
-  if (sUseLongTapInjector) {
+  if (StaticPrefs::layout_accessiblecaret_use_long_tap_injector()) {
     mLongTapInjectorTimer = NS_NewTimer();
   }
 
@@ -353,10 +372,9 @@ void AccessibleCaretEventHub::Terminate() {
     return;
   }
 
-  RefPtr<nsDocShell> docShell(mDocShell.get());
-  if (docShell) {
-    docShell->RemoveWeakReflowObserver(this);
-    docShell->RemoveWeakScrollObserver(this);
+  if (mDocShell) {
+    mDocShell->RemoveWeakReflowObserver(this);
+    mDocShell->RemoveWeakScrollObserver(this);
   }
 
   if (mLongTapInjectorTimer) {
@@ -378,6 +396,11 @@ nsEventStatus AccessibleCaretEventHub::HandleEvent(WidgetEvent* aEvent) {
   MOZ_ASSERT(mRefCnt.get() > 1, "Expect caller holds us as well!");
 
   switch (aEvent->mClass) {
+    case ePointerEventClass:
+      if (!IsPointerEventMessageOriginallyMouseEventMessage(aEvent->mMessage)) {
+        break;
+      }
+      [[fallthrough]];
     case eMouseEventClass:
       status = HandleMouseEvent(aEvent->AsMouseEvent());
       break;
@@ -391,6 +414,8 @@ nsEventStatus AccessibleCaretEventHub::HandleEvent(WidgetEvent* aEvent) {
       break;
 
     default:
+      MOZ_ASSERT_UNREACHABLE(
+          "PresShell should've filtered unwanted event classes!");
       break;
   }
 
@@ -401,7 +426,7 @@ nsEventStatus AccessibleCaretEventHub::HandleMouseEvent(
     WidgetMouseEvent* aEvent) {
   nsEventStatus rv = nsEventStatus_eIgnore;
 
-  if (aEvent->button != WidgetMouseEvent::eLeftButton) {
+  if (aEvent->mButton != MouseButton::ePrimary) {
     return rv;
   }
 
@@ -410,12 +435,12 @@ nsEventStatus AccessibleCaretEventHub::HandleMouseEvent(
   nsPoint point = GetMouseEventPosition(aEvent);
 
   if (aEvent->mMessage == eMouseDown || aEvent->mMessage == eMouseUp ||
-      aEvent->mMessage == eMouseClick ||
+      aEvent->mMessage == ePointerClick ||
       aEvent->mMessage == eMouseDoubleClick ||
       aEvent->mMessage == eMouseLongTap) {
     // Don't reset the source on mouse movement since that can
     // happen anytime, even randomly during a touch sequence.
-    mManager->SetLastInputSource(aEvent->inputSource);
+    mManager->SetLastInputSource(aEvent->mInputSource);
   }
 
   switch (aEvent->mMessage) {
@@ -427,7 +452,10 @@ nsEventStatus AccessibleCaretEventHub::HandleMouseEvent(
 
     case eMouseMove:
       AC_LOGV("Before eMouseMove, state: %s", mState->Name());
-      rv = mState->OnMove(this, point);
+      // The mouse move events synthesized from the touch move events can have
+      // wrong point (bug 1549355). Workaround it by ignoring the events when
+      // dragging the caret because the caret doesn't really need them.
+      rv = mState->OnMove(this, point, aEvent->mReason);
       AC_LOGV("After eMouseMove, state: %s, consume: %d", mState->Name(), rv);
       break;
 
@@ -465,7 +493,7 @@ nsEventStatus AccessibleCaretEventHub::HandleTouchEvent(
                                          : mActiveTouchId);
   nsPoint point = GetTouchEventPosition(aEvent, id);
 
-  mManager->SetLastInputSource(nsIDOMMouseEvent::MOZ_SOURCE_TOUCH);
+  mManager->SetLastInputSource(MouseEvent_Binding::MOZ_SOURCE_TOUCH);
 
   switch (aEvent->mMessage) {
     case eTouchStart:
@@ -476,7 +504,8 @@ nsEventStatus AccessibleCaretEventHub::HandleTouchEvent(
 
     case eTouchMove:
       AC_LOGV("Before eTouchMove, state: %s", mState->Name());
-      rv = mState->OnMove(this, point);
+      // There is no synthesized touch move event.
+      rv = mState->OnMove(this, point, WidgetMouseEvent::eReal);
       AC_LOGV("After eTouchMove, state: %s, consume: %d", mState->Name(), rv);
       break;
 
@@ -500,7 +529,7 @@ nsEventStatus AccessibleCaretEventHub::HandleTouchEvent(
 
 nsEventStatus AccessibleCaretEventHub::HandleKeyboardEvent(
     WidgetKeyboardEvent* aEvent) {
-  mManager->SetLastInputSource(nsIDOMMouseEvent::MOZ_SOURCE_KEYBOARD);
+  mManager->SetLastInputSource(MouseEvent_Binding::MOZ_SOURCE_KEYBOARD);
 
   switch (aEvent->mMessage) {
     case eKeyUp:
@@ -528,7 +557,7 @@ nsEventStatus AccessibleCaretEventHub::HandleKeyboardEvent(
 bool AccessibleCaretEventHub::MoveDistanceIsLarge(const nsPoint& aPoint) const {
   nsPoint delta = aPoint - mPressPoint;
   return NS_hypot(delta.x, delta.y) >
-         nsPresContext::AppUnitsPerCSSPixel() * kMoveStartToleranceInPixel;
+         AppUnitsPerCSSPixel() * kMoveStartToleranceInPixel;
 }
 
 void AccessibleCaretEventHub::LaunchLongTapInjector() {
@@ -536,7 +565,7 @@ void AccessibleCaretEventHub::LaunchLongTapInjector() {
     return;
   }
 
-  int32_t longTapDelay = gfxPrefs::UiClickHoldContextMenusDelay();
+  int32_t longTapDelay = StaticPrefs::ui_click_hold_context_menus_delay();
   mLongTapInjectorTimer->InitWithNamedFuncCallback(
       FireLongTap, this, longTapDelay, nsITimer::TYPE_ONE_SHOT,
       "AccessibleCaretEventHub::LaunchLongTapInjector");
@@ -550,8 +579,9 @@ void AccessibleCaretEventHub::CancelLongTapInjector() {
   mLongTapInjectorTimer->Cancel();
 }
 
-/* static */ void AccessibleCaretEventHub::FireLongTap(
-    nsITimer* aTimer, void* aAccessibleCaretEventHub) {
+/* static */
+void AccessibleCaretEventHub::FireLongTap(nsITimer* aTimer,
+                                          void* aAccessibleCaretEventHub) {
   RefPtr<AccessibleCaretEventHub> self =
       static_cast<AccessibleCaretEventHub*>(aAccessibleCaretEventHub);
   self->mState->OnLongTap(self, self->mPressPoint);
@@ -618,18 +648,24 @@ void AccessibleCaretEventHub::ScrollPositionChanged() {
   mState->OnScrollPositionChanged(this);
 }
 
-nsresult AccessibleCaretEventHub::NotifySelectionChanged(nsIDOMDocument* aDoc,
-                                                         nsISelection* aSel,
-                                                         int16_t aReason) {
+void AccessibleCaretEventHub::OnSelectionChange(Document* aDoc,
+                                                dom::Selection* aSel,
+                                                int16_t aReason) {
   if (!mInitialized) {
-    return NS_OK;
+    return;
   }
 
   MOZ_ASSERT(mRefCnt.get() > 1, "Expect caller holds us as well!");
 
   AC_LOG("%s, state: %s, reason: %d", __FUNCTION__, mState->Name(), aReason);
+
+  // XXX Here we may be in a hot path.  So, if we could avoid this virtual call,
+  //     we should do so.
   mState->OnSelectionChanged(this, aDoc, aSel, aReason);
-  return NS_OK;
+}
+
+bool AccessibleCaretEventHub::ShouldDisableApz() const {
+  return mManager && mManager->ShouldDisableApz();
 }
 
 void AccessibleCaretEventHub::NotifyBlur(bool aIsLeavingDocument) {
@@ -651,8 +687,8 @@ nsPoint AccessibleCaretEventHub::GetTouchEventPosition(
 
       // Get event coordinate relative to root frame.
       nsIFrame* rootFrame = mPresShell->GetRootFrame();
-      return nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent, touchIntPoint,
-                                                          rootFrame);
+      return nsLayoutUtils::GetEventCoordinatesRelativeTo(
+          aEvent, touchIntPoint, RelativeTo{rootFrame});
     }
   }
   return nsPoint(NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE);
@@ -665,7 +701,7 @@ nsPoint AccessibleCaretEventHub::GetMouseEventPosition(
   // Get event coordinate relative to root frame.
   nsIFrame* rootFrame = mPresShell->GetRootFrame();
   return nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent, mouseIntPoint,
-                                                      rootFrame);
+                                                      RelativeTo{rootFrame});
 }
 
 }  // namespace mozilla
