@@ -7,9 +7,16 @@
 
 #include "TouchManager.h"
 
+#include "Units.h"
+#include "mozilla/EventForwards.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_apz.h"
+#include "mozilla/StaticPrefs_test.h"
+#include "mozilla/TimeStamp.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/EventTarget.h"
-#include "mozilla/PresShell.h"
+#include "mozilla/dom/PointerEventHandler.h"
+#include "mozilla/layers/InputAPZContext.h"
 #include "nsIContent.h"
 #include "nsIFrame.h"
 #include "nsLayoutUtils.h"
@@ -23,6 +30,9 @@ namespace mozilla {
 StaticAutoPtr<nsTHashMap<nsUint32HashKey, TouchManager::TouchInfo>>
     TouchManager::sCaptureTouchList;
 layers::LayersId TouchManager::sCaptureTouchLayersId;
+TimeStamp TouchManager::sSingleTouchStartTimeStamp;
+LayoutDeviceIntPoint TouchManager::sSingleTouchStartPoint;
+bool TouchManager::sPrecedingTouchPointerDownConsumedByContent = false;
 
 /*static*/
 void TouchManager::InitializeStatics() {
@@ -68,11 +78,10 @@ void TouchManager::EvictTouchPoint(RefPtr<Touch>& aTouch,
       if (presShell) {
         nsIFrame* frame = presShell->GetRootFrame();
         if (frame) {
-          nsPoint pt(aTouch->mRefPoint.x, aTouch->mRefPoint.y);
-          nsCOMPtr<nsIWidget> widget = frame->GetView()->GetNearestWidget(&pt);
+          nsCOMPtr<nsIWidget> widget =
+              frame->GetView()->GetNearestWidget(nullptr);
           if (widget) {
             WidgetTouchEvent event(true, eTouchEnd, widget);
-            event.mTime = PR_IntervalNow();
             event.mTouches.AppendElement(aTouch);
             nsEventStatus status;
             widget->DispatchEvent(&event, status);
@@ -128,8 +137,7 @@ nsIFrame* TouchManager::SetupTarget(WidgetTouchEvent* aEvent,
           aEvent, touch->mRefPoint, relativeTo);
       target = FindFrameTargetedByInputEvent(aEvent, relativeTo, eventPoint);
       if (target) {
-        nsCOMPtr<nsIContent> targetContent;
-        target->GetContentForEvent(aEvent, getter_AddRefs(targetContent));
+        nsIContent* targetContent = target->GetContentForEvent(aEvent);
         touch->SetTouchTarget(targetContent
                                   ? targetContent->GetAsElementOrParentElement()
                                   : nullptr);
@@ -168,7 +176,7 @@ nsIFrame* TouchManager::SuppressInvalidPointsAndGetTargetedFrame(
   }
 
   nsIFrame* frame = nullptr;
-  for (int32_t i = aEvent->mTouches.Length(); i;) {
+  for (uint32_t i = aEvent->mTouches.Length(); i;) {
     --i;
     dom::Touch* touch = aEvent->mTouches[i];
     if (TouchManager::HasCapturedTouch(touch->Identifier())) {
@@ -176,9 +184,35 @@ nsIFrame* TouchManager::SuppressInvalidPointsAndGetTargetedFrame(
     }
 
     MOZ_ASSERT(touch->mOriginalTarget);
-    nsCOMPtr<nsIContent> targetContent = do_QueryInterface(touch->GetTarget());
-    nsIFrame* targetFrame =
-        targetContent ? targetContent->GetPrimaryFrame() : nullptr;
+    nsIContent* const targetContent =
+        nsIContent::FromEventTargetOrNull(touch->GetTarget());
+    if (MOZ_UNLIKELY(!targetContent)) {
+      touch->mIsTouchEventSuppressed = true;
+      continue;
+    }
+
+    // Even if the target content is not connected, we should dispatch the touch
+    // start event except when the target content is owned by different
+    // document.
+    if (MOZ_UNLIKELY(!targetContent->IsInComposedDoc())) {
+      if (anyTarget && anyTarget->OwnerDoc() != targetContent->OwnerDoc()) {
+        touch->mIsTouchEventSuppressed = true;
+        continue;
+      }
+      if (!anyTarget) {
+        anyTarget = targetContent;
+      }
+      touch->SetTouchTarget(targetContent->GetAsElementOrParentElement());
+      if (PresShell* const presShell =
+              targetContent->OwnerDoc()->GetPresShell()) {
+        if (nsIFrame* rootFrame = presShell->GetRootFrame()) {
+          frame = rootFrame;
+        }
+      }
+      continue;
+    }
+
+    nsIFrame* targetFrame = targetContent->GetPrimaryFrame();
     if (targetFrame && !anyTarget) {
       anyTarget = targetContent;
     } else {
@@ -202,10 +236,10 @@ nsIFrame* TouchManager::SuppressInvalidPointsAndGetTargetedFrame(
         touch->mIsTouchEventSuppressed = true;
       } else {
         targetFrame = newTargetFrame;
-        targetFrame->GetContentForEvent(aEvent, getter_AddRefs(targetContent));
-        touch->SetTouchTarget(targetContent
-                                  ? targetContent->GetAsElementOrParentElement()
-                                  : nullptr);
+        nsIContent* newTargetContent = targetFrame->GetContentForEvent(aEvent);
+        touch->SetTouchTarget(
+            newTargetContent ? newTargetContent->GetAsElementOrParentElement()
+                             : nullptr);
       }
     }
     if (targetFrame) {
@@ -236,8 +270,15 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
         // touch event associated to. We cache layers id of the first touchstart
         // event, all subsequent touch events will use the same layers id.
         sCaptureTouchLayersId = aEvent->mLayersId;
+        sSingleTouchStartTimeStamp = aEvent->mTimeStamp;
+        sSingleTouchStartPoint = touchEvent->mTouches[0]->mRefPoint;
+        const PointerInfo* pointerInfo = PointerEventHandler::GetPointerInfo(
+            touchEvent->mTouches[0]->Identifier());
+        sPrecedingTouchPointerDownConsumedByContent =
+            pointerInfo && pointerInfo->mPreventMouseEventByContent;
       } else {
         touchEvent->mLayersId = sCaptureTouchLayersId;
+        sSingleTouchStartTimeStamp = TimeStamp();
       }
       // Add any new touches to the queue
       WidgetTouchEvent::TouchArray& touches = touchEvent->mTouches;
@@ -262,6 +303,9 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
       }
       break;
     }
+    case eTouchRawUpdate:
+      MOZ_ASSERT_UNREACHABLE("eTouchRawUpdate shouldn't be handled as a touch");
+      break;
     case eTouchMove: {
       // Check for touches that changed. Mark them add to queue
       WidgetTouchEvent* touchEvent = aEvent->AsTouchEvent();
@@ -329,6 +373,10 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
             }
           }
         } else {
+          // This touch event isn't going to be dispatched on the main-thread,
+          // we need to tell it to APZ because returned nsEventStatus is
+          // unreliable to tell whether the event was preventDefaulted or not.
+          layers::InputAPZContext::SetDropped();
           return false;
         }
       }
@@ -400,6 +448,63 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
   return true;
 }
 
+void TouchManager::PostHandleEvent(const WidgetEvent* aEvent,
+                                   const nsEventStatus* aStatus) {
+  switch (aEvent->mMessage) {
+    case eTouchRawUpdate:
+      MOZ_ASSERT_UNREACHABLE("eTouchRawUpdate shouldn't be handled as a touch");
+      break;
+    case eTouchMove: {
+      if (sSingleTouchStartTimeStamp.IsNull()) {
+        break;
+      }
+      if (*aStatus == nsEventStatus_eConsumeNoDefault) {
+        sSingleTouchStartTimeStamp = TimeStamp();
+        break;
+      }
+      const WidgetTouchEvent* touchEvent = aEvent->AsTouchEvent();
+      if (touchEvent->mTouches.Length() > 1) {
+        sSingleTouchStartTimeStamp = TimeStamp();
+        break;
+      }
+      if (touchEvent->mTouches.Length() == 1) {
+        // If the touch moved too far from the start point, don't treat the
+        // touch as a tap.
+        const float distance =
+            static_cast<float>((sSingleTouchStartPoint -
+                                aEvent->AsTouchEvent()->mTouches[0]->mRefPoint)
+                                   .Length());
+        const float maxDistance =
+            StaticPrefs::apz_touch_start_tolerance() *
+            (MOZ_LIKELY(touchEvent->mWidget) ? touchEvent->mWidget->GetDPI()
+                                             : 96.0f);
+        if (distance > maxDistance) {
+          sSingleTouchStartTimeStamp = TimeStamp();
+        }
+      }
+      break;
+    }
+    case eTouchStart:
+    case eTouchEnd:
+      if (*aStatus == nsEventStatus_eConsumeNoDefault &&
+          !sSingleTouchStartTimeStamp.IsNull()) {
+        sSingleTouchStartTimeStamp = TimeStamp();
+      }
+      break;
+    case eTouchCancel:
+    case eTouchPointerCancel:
+    case eMouseLongTap:
+    case eContextMenu: {
+      if (!sSingleTouchStartTimeStamp.IsNull()) {
+        sSingleTouchStartTimeStamp = TimeStamp();
+      }
+      break;
+    }
+    default:
+      break;
+  }
+}
+
 /*static*/
 already_AddRefed<nsIContent> TouchManager::GetAnyCapturedTouchTarget() {
   nsCOMPtr<nsIContent> result = nullptr;
@@ -460,10 +565,9 @@ bool TouchManager::ShouldConvertTouchToPointer(const Touch* aTouch,
       // We don't want to fire duplicated pointerdown.
       return false;
     }
-    case eTouchMove: {
-      // Always fire first pointermove event.
-      return info.mTouch->mMessage != eTouchMove ||
-             !aTouch->Equals(info.mTouch);
+    case eTouchMove:
+    case eTouchRawUpdate: {
+      return !aTouch->Equals(info.mTouch);
     }
     default:
       break;
@@ -471,4 +575,30 @@ bool TouchManager::ShouldConvertTouchToPointer(const Touch* aTouch,
   return true;
 }
 
+/* static */
+bool TouchManager::IsSingleTapEndToDoDefault(
+    const WidgetTouchEvent* aTouchEndEvent) {
+  MOZ_ASSERT(aTouchEndEvent);
+  MOZ_ASSERT(aTouchEndEvent->mFlags.mIsSynthesizedForTests);
+  MOZ_ASSERT(!StaticPrefs::test_events_async_enabled());
+  if (sSingleTouchStartTimeStamp.IsNull() ||
+      aTouchEndEvent->mTouches.Length() != 1) {
+    return false;
+  }
+  // If it's pressed long time, we should not treat it as a single tap because
+  // a long press should cause opening context menu by default.
+  if ((aTouchEndEvent->mTimeStamp - sSingleTouchStartTimeStamp)
+          .ToMilliseconds() > StaticPrefs::apz_max_tap_time()) {
+    return false;
+  }
+  NS_WARNING_ASSERTION(aTouchEndEvent->mTouches[0]->mChanged,
+                       "The single tap end should be changed");
+  return true;
+}
+
+/* static */
+bool TouchManager::IsPrecedingTouchPointerDownConsumedByContent() {
+  return sPrecedingTouchPointerDownConsumedByContent;
+}
+
 }  // namespace mozilla
