@@ -7,15 +7,20 @@
 #ifndef ZoomConstraintsClient_h_
 #define ZoomConstraintsClient_h_
 
-#include "FrameMetrics.h"
+#include "mozilla/layers/ScrollableLayerGuid.h"
+#include "mozilla/layers/ZoomConstraints.h"
 #include "mozilla/Maybe.h"
+#include "nsCOMPtr.h"
 #include "nsIDOMEventListener.h"
 #include "nsIObserver.h"
-#include "nsWeakPtr.h"
 
-class nsIDOMEventTarget;
-class nsIDocument;
-class nsIPresShell;
+namespace mozilla {
+class PresShell;
+namespace dom {
+class Document;
+class EventTarget;
+}  // namespace dom
+}  // namespace mozilla
 
 class ZoomConstraintsClient final : public nsIDOMEventListener,
                                     public nsIObserver {
@@ -30,18 +35,21 @@ class ZoomConstraintsClient final : public nsIDOMEventListener,
   ~ZoomConstraintsClient();
 
  public:
-  void Init(nsIPresShell* aPresShell, nsIDocument* aDocument);
+  void Init(mozilla::PresShell* aPresShell, mozilla::dom::Document* aDocument);
   void Destroy();
   void ScreenSizeChanged();
 
+  bool GetAllowZoom() const { return mZoomConstraints.mAllowZoom; }
+
  private:
   void RefreshZoomConstraints();
 
-  nsCOMPtr<nsIDocument> mDocument;
-  nsIPresShell* MOZ_NON_OWNING_REF
-      mPresShell;  // raw ref since the presShell owns this
-  nsCOMPtr<nsIDOMEventTarget> mEventTarget;
+  RefPtr<mozilla::dom::Document> mDocument;
+  // raw ref since the presShell owns this
+  mozilla::PresShell* MOZ_NON_OWNING_REF mPresShell;
+  nsCOMPtr<mozilla::dom::EventTarget> mEventTarget;
   mozilla::Maybe<mozilla::layers::ScrollableLayerGuid> mGuid;
+  mozilla::layers::ZoomConstraints mZoomConstraints;
 };
 
 #endif
