@@ -19,11 +19,7 @@ nsBlockFrame* NS_NewColumnSetWrapperFrame(PresShell* aPresShell,
                                           nsFrameState aStateFlags) {
   ColumnSetWrapperFrame* frame = new (aPresShell)
       ColumnSetWrapperFrame(aStyle, aPresShell->GetPresContext());
-
-  // CSS Multi-column level 1 section 2: A multi-column container
-  // establishes a new block formatting context, as per CSS 2.1 section
-  // 9.4.1.
-  frame->AddStateBits(aStateFlags | NS_BLOCK_FORMATTING_CONTEXT_STATE_BITS);
+  frame->AddStateBits(aStateFlags);
   return frame;
 }
 
@@ -77,7 +73,8 @@ void ColumnSetWrapperFrame::AppendDirectlyOwnedAnonBoxes(
   // asserts all the conditions above which allow us to skip appending
   // -moz-column-span-wrappers.
   auto FindFirstChildInChildLists = [this]() -> nsIFrame* {
-    const ChildListID listIDs[] = {kPrincipalList, kOverflowList};
+    const ChildListID listIDs[] = {FrameChildListID::Principal,
+                                   FrameChildListID::Overflow};
     for (nsIFrame* frag = this; frag; frag = frag->GetNextInFlow()) {
       for (ChildListID id : listIDs) {
         const nsFrameList& list = frag->GetChildList(id);
@@ -105,13 +102,13 @@ nsresult ColumnSetWrapperFrame::GetFrameName(nsAString& aResult) const {
 // column hierarchy since any change to the column hierarchy in the column
 // sub-tree need to be re-created.
 void ColumnSetWrapperFrame::AppendFrames(ChildListID aListID,
-                                         nsFrameList& aFrameList) {
+                                         nsFrameList&& aFrameList) {
 #ifdef DEBUG
   MOZ_ASSERT(!mFinishedBuildingColumns, "Should only call once!");
   mFinishedBuildingColumns = true;
 #endif
 
-  nsBlockFrame::AppendFrames(aListID, aFrameList);
+  nsBlockFrame::AppendFrames(aListID, std::move(aFrameList));
 
 #ifdef DEBUG
   nsIFrame* firstColumnSet = PrincipalChildList().FirstChild();
@@ -129,15 +126,17 @@ void ColumnSetWrapperFrame::AppendFrames(ChildListID aListID,
 
 void ColumnSetWrapperFrame::InsertFrames(
     ChildListID aListID, nsIFrame* aPrevFrame,
-    const nsLineList::iterator* aPrevFrameLine, nsFrameList& aFrameList) {
+    const nsLineList::iterator* aPrevFrameLine, nsFrameList&& aFrameList) {
   MOZ_ASSERT_UNREACHABLE("Unsupported operation!");
-  nsBlockFrame::InsertFrames(aListID, aPrevFrame, aPrevFrameLine, aFrameList);
+  nsBlockFrame::InsertFrames(aListID, aPrevFrame, aPrevFrameLine,
+                             std::move(aFrameList));
 }
 
-void ColumnSetWrapperFrame::RemoveFrame(ChildListID aListID,
+void ColumnSetWrapperFrame::RemoveFrame(DestroyContext& aContext,
+                                        ChildListID aListID,
                                         nsIFrame* aOldFrame) {
   MOZ_ASSERT_UNREACHABLE("Unsupported operation!");
-  nsBlockFrame::RemoveFrame(aListID, aOldFrame);
+  nsBlockFrame::RemoveFrame(aContext, aListID, aOldFrame);
 }
 
 void ColumnSetWrapperFrame::MarkIntrinsicISizesDirty() {
@@ -150,15 +149,29 @@ void ColumnSetWrapperFrame::MarkIntrinsicISizesDirty() {
   }
 }
 
-nscoord ColumnSetWrapperFrame::GetMinISize(gfxContext* aRenderingContext) {
-  nscoord iSize = 0;
-  DISPLAY_MIN_INLINE_SIZE(this, iSize);
+nscoord ColumnSetWrapperFrame::IntrinsicISize(const IntrinsicSizeInput& aInput,
+                                              IntrinsicISizeType aType) {
+  return mCachedIntrinsics.GetOrSet(*this, aType, aInput, [&] {
+    return aType == IntrinsicISizeType::MinISize ? MinISize(aInput)
+                                                 : PrefISize(aInput);
+  });
+}
 
-  if (StyleDisplay()->GetContainSizeAxes().mIContained) {
-    // If we're size-contained in inline axis, we determine our minimum
-    // intrinsic size purely from our column styling, as if we had no
-    // descendants. This should match what happens in
-    // nsColumnSetFrame::GetMinISize in an actual no-descendants scenario.
+nscoord ColumnSetWrapperFrame::MinISize(const IntrinsicSizeInput& aInput) {
+  nscoord iSize = 0;
+
+  if (Maybe<nscoord> containISize =
+          ContainIntrinsicISize(NS_UNCONSTRAINEDSIZE)) {
+    // If we're size-contained in inline axis and contain-intrinsic-inline-size
+    // is not 'none', then use that size.
+    if (*containISize != NS_UNCONSTRAINEDSIZE) {
+      return *containISize;
+    }
+
+    // In the 'none' case, we determine our minimum intrinsic size purely from
+    // our column styling, as if we had no descendants. This should match what
+    // happens in nsColumnSetFrame::MinISize in an actual no-descendants
+    // scenario.
     const nsStyleColumn* colStyle = StyleColumn();
     if (colStyle->mColumnWidth.IsLength()) {
       // As available inline size reduces to zero, our number of columns reduces
@@ -167,57 +180,117 @@ nscoord ColumnSetWrapperFrame::GetMinISize(gfxContext* aRenderingContext) {
       // size, either. Just use 0 because we're size-contained.
       iSize = 0;
     } else {
-      MOZ_ASSERT(colStyle->mColumnCount != nsStyleColumn::kColumnCountAuto,
+      MOZ_ASSERT(!colStyle->mColumnCount.IsAuto(),
                  "column-count and column-width can't both be auto!");
       // As available inline size reduces to zero, we still have mColumnCount
       // columns, so compute our minimum intrinsic size based on N zero-width
       // columns, with specified gap size between them.
       const nscoord colGap =
           ColumnUtils::GetColumnGap(this, NS_UNCONSTRAINEDSIZE);
-      iSize = ColumnUtils::IntrinsicISize(colStyle->mColumnCount, colGap, 0);
+      iSize = ColumnUtils::IntrinsicISize(colStyle->mColumnCount.AsInteger(),
+                                          colGap, 0);
     }
   } else {
     for (nsIFrame* f : PrincipalChildList()) {
-      iSize = std::max(iSize, f->GetMinISize(aRenderingContext));
+      const IntrinsicSizeInput childInput(aInput, f->GetWritingMode(),
+                                          GetWritingMode());
+      iSize = std::max(iSize, f->GetMinISize(childInput));
     }
   }
 
   return iSize;
 }
 
-nscoord ColumnSetWrapperFrame::GetPrefISize(gfxContext* aRenderingContext) {
+nscoord ColumnSetWrapperFrame::PrefISize(const IntrinsicSizeInput& aInput) {
   nscoord iSize = 0;
-  DISPLAY_PREF_INLINE_SIZE(this, iSize);
 
-  if (StyleDisplay()->GetContainSizeAxes().mIContained) {
+  if (Maybe<nscoord> containISize =
+          ContainIntrinsicISize(NS_UNCONSTRAINEDSIZE)) {
+    if (*containISize != NS_UNCONSTRAINEDSIZE) {
+      return *containISize;
+    }
+
     const nsStyleColumn* colStyle = StyleColumn();
     nscoord colISize;
     if (colStyle->mColumnWidth.IsLength()) {
       colISize =
           ColumnUtils::ClampUsedColumnWidth(colStyle->mColumnWidth.AsLength());
     } else {
-      MOZ_ASSERT(colStyle->mColumnCount != nsStyleColumn::kColumnCountAuto,
+      MOZ_ASSERT(!colStyle->mColumnCount.IsAuto(),
                  "column-count and column-width can't both be auto!");
       colISize = 0;
     }
 
     // If column-count is auto, assume one column.
-    const uint32_t numColumns =
-        colStyle->mColumnCount == nsStyleColumn::kColumnCountAuto
-            ? 1
-            : colStyle->mColumnCount;
+    const uint32_t numColumns = colStyle->mColumnCount.IsAuto()
+                                    ? 1
+                                    : colStyle->mColumnCount.AsInteger();
     const nscoord colGap =
         ColumnUtils::GetColumnGap(this, NS_UNCONSTRAINEDSIZE);
     iSize = ColumnUtils::IntrinsicISize(numColumns, colGap, colISize);
   } else {
     for (nsIFrame* f : PrincipalChildList()) {
-      iSize = std::max(iSize, f->GetPrefISize(aRenderingContext));
+      const IntrinsicSizeInput childInput(aInput, f->GetWritingMode(),
+                                          GetWritingMode());
+      iSize = std::max(iSize, f->GetPrefISize(childInput));
     }
   }
 
   return iSize;
 }
 
+template <typename Iterator>
+Maybe<nscoord> ColumnSetWrapperFrame::GetBaselineBOffset(
+    Iterator aStart, Iterator aEnd, WritingMode aWM,
+    BaselineSharingGroup aBaselineGroup,
+    BaselineExportContext aExportContext) const {
+  // Either forward iterator + first baseline, or reverse iterator + last
+  // baseline
+  MOZ_ASSERT((*aStart == PrincipalChildList().FirstChild() &&
+              aBaselineGroup == BaselineSharingGroup::First) ||
+                 (*aStart == PrincipalChildList().LastChild() &&
+                  aBaselineGroup == BaselineSharingGroup::Last),
+             "Iterator direction must match baseline sharing group.");
+  if (StyleDisplay()->IsContainLayout()) {
+    return Nothing{};
+  }
+
+  // Start from start/end of principal child list, and use the first valid
+  // baseline.
+  for (auto itr = aStart; itr != aEnd; ++itr) {
+    const nsIFrame* kid = *itr;
+    auto kidBaseline =
+        kid->GetNaturalBaselineBOffset(aWM, aBaselineGroup, aExportContext);
+    if (!kidBaseline) {
+      continue;
+    }
+    // Baseline is offset from the kid's rectangle, so find the offset to the
+    // kid's rectangle.
+    LogicalRect kidRect{aWM, kid->GetLogicalNormalPosition(aWM, GetSize()),
+                        kid->GetLogicalSize(aWM)};
+    if (aBaselineGroup == BaselineSharingGroup::First) {
+      *kidBaseline += kidRect.BStart(aWM);
+    } else {
+      *kidBaseline += (GetLogicalSize().BSize(aWM) - kidRect.BEnd(aWM));
+    }
+    return kidBaseline;
+  }
+  return Nothing{};
+}
+
+Maybe<nscoord> ColumnSetWrapperFrame::GetNaturalBaselineBOffset(
+    WritingMode aWM, BaselineSharingGroup aBaselineGroup,
+    BaselineExportContext aExportContext) const {
+  if (aBaselineGroup == BaselineSharingGroup::First) {
+    return GetBaselineBOffset(PrincipalChildList().cbegin(),
+                              PrincipalChildList().cend(), aWM, aBaselineGroup,
+                              aExportContext);
+  }
+  return GetBaselineBOffset(PrincipalChildList().crbegin(),
+                            PrincipalChildList().crend(), aWM, aBaselineGroup,
+                            aExportContext);
+}
+
 #ifdef DEBUG
 
 /* static */
