@@ -9,30 +9,28 @@
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsExceptionHandler.h"
 #include "mozilla/MemoryReporting.h"
+#include "mozilla/NeverDestroyed.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/StaticPrefs_layout.h"
 #include "mozilla/StyleSheet.h"
 #include "mozilla/StyleSheetInlines.h"
-#include "mozilla/Telemetry.h"
 #include "mozilla/css/Loader.h"
-#include "mozilla/StaticPrefs_browser.h"
+#include "mozilla/dom/ReferrerInfo.h"
 #include "mozilla/dom/SRIMetadata.h"
-#include "mozilla/ipc/SharedMemory.h"
+#include "mozilla/ipc/SharedMemoryHandle.h"
+#include "mozilla/ipc/SharedMemoryMapping.h"
+#include "mozilla/ServoBindings.h"
 #include "MainThreadUtils.h"
-#include "nsColor.h"
 #include "nsContentUtils.h"
 #include "nsIConsoleService.h"
 #include "nsIFile.h"
 #include "nsIObserverService.h"
 #include "nsIXULRuntime.h"
 #include "nsNetUtil.h"
-#include "nsPresContext.h"
 #include "nsPrintfCString.h"
 #include "nsServiceManagerUtils.h"
 #include "nsXULAppAPI.h"
 
-#include <mozilla/ServoBindings.h>
-
 namespace mozilla {
 
 // The GlobalStyleSheetCache is responsible for sharing user agent style sheet
@@ -107,6 +105,11 @@ namespace mozilla {
 using namespace mozilla;
 using namespace css;
 
+mozilla::ipc::ReadOnlySharedMemoryHandle& sSharedMemoryHandle() {
+  static NeverDestroyed<mozilla::ipc::ReadOnlySharedMemoryHandle> handle;
+  return *handle;
+}
+
 #define PREF_LEGACY_STYLESHEET_CUSTOMIZATION \
   "toolkit.legacyUserProfileCustomizations.stylesheets"
 
@@ -126,15 +129,30 @@ nsresult GlobalStyleSheetCache::Observe(nsISupports* aSubject,
   return NS_OK;
 }
 
-#define STYLE_SHEET(identifier_, url_, shared_)                                \
-  NotNull<StyleSheet*> GlobalStyleSheetCache::identifier_##Sheet() {           \
-    if (!m##identifier_##Sheet) {                                              \
-      m##identifier_##Sheet = LoadSheetURL(url_, eAgentSheetFeatures, eCrash); \
-    }                                                                          \
-    return WrapNotNull(m##identifier_##Sheet);                                 \
-  }
-#include "mozilla/UserAgentStyleSheetList.h"
+static constexpr struct {
+  nsLiteralCString mURL;
+  BuiltInStyleSheetFlags mFlags;
+} kBuiltInSheetInfo[] = {
+#define STYLE_SHEET(identifier_, url_, flags_) \
+  {nsLiteralCString(url_), BuiltInStyleSheetFlags::flags_},
+#include "mozilla/BuiltInStyleSheetList.h"
 #undef STYLE_SHEET
+};
+
+NotNull<StyleSheet*> GlobalStyleSheetCache::BuiltInSheet(
+    BuiltInStyleSheet aSheet) {
+  auto& slot = mBuiltIns[aSheet];
+  if (!slot) {
+    const auto& info = kBuiltInSheetInfo[size_t(aSheet)];
+    const auto parsingMode = (info.mFlags & BuiltInStyleSheetFlags::UA)
+                                 ? eAgentSheetFeatures
+                                 : eAuthorSheetFeatures;
+    MOZ_ASSERT(info.mFlags & BuiltInStyleSheetFlags::UA ||
+               info.mFlags & BuiltInStyleSheetFlags::Author);
+    slot = LoadSheetURL(info.mURL, parsingMode, eCrash);
+  }
+  return WrapNotNull(slot);
+}
 
 StyleSheet* GlobalStyleSheetCache::GetUserContentSheet() {
   return mUserContentSheet;
@@ -144,24 +162,6 @@ StyleSheet* GlobalStyleSheetCache::GetUserChromeSheet() {
   return mUserChromeSheet;
 }
 
-StyleSheet* GlobalStyleSheetCache::ChromePreferenceSheet() {
-  if (!mChromePreferenceSheet) {
-    BuildPreferenceSheet(&mChromePreferenceSheet,
-                         PreferenceSheet::ChromePrefs());
-  }
-
-  return mChromePreferenceSheet;
-}
-
-StyleSheet* GlobalStyleSheetCache::ContentPreferenceSheet() {
-  if (!mContentPreferenceSheet) {
-    BuildPreferenceSheet(&mContentPreferenceSheet,
-                         PreferenceSheet::ContentPrefs());
-  }
-
-  return mContentPreferenceSheet;
-}
-
 void GlobalStyleSheetCache::Shutdown() {
   gCSSLoader = nullptr;
   NS_WARNING_ASSERTION(!gStyleCache || !gUserContentSheetURL,
@@ -194,7 +194,7 @@ GlobalStyleSheetCache::CollectReports(nsIHandleReportCallback* aHandleReport,
   if (XRE_IsParentProcess()) {
     MOZ_COLLECT_REPORT(
         "explicit/layout/style-sheet-cache/shared", KIND_NONHEAP, UNITS_BYTES,
-        sSharedMemory ? sUsedSharedMemory : 0,
+        sSharedMemory.IsEmpty() ? 0 : sUsedSharedMemory,
         "Memory used for built-in style sheets that are shared to "
         "child processes.");
   }
@@ -208,12 +208,10 @@ size_t GlobalStyleSheetCache::SizeOfIncludingThis(
 
 #define MEASURE(s) n += s ? s->SizeOfIncludingThis(aMallocSizeOf) : 0;
 
-#define STYLE_SHEET(identifier_, url_, shared_) MEASURE(m##identifier_##Sheet);
-#include "mozilla/UserAgentStyleSheetList.h"
-#undef STYLE_SHEET
+  for (const auto& sheet : mBuiltIns) {
+    MEASURE(sheet);
+  }
 
-  MEASURE(mChromePreferenceSheet);
-  MEASURE(mContentPreferenceSheet);
   MEASURE(mUserChromeSheet);
   MEASURE(mUserContentSheet);
 
@@ -259,10 +257,10 @@ GlobalStyleSheetCache::GlobalStyleSheetCache() {
     if (XRE_IsParentProcess()) {
       // Load the style sheets and store them in a new shared memory buffer.
       InitSharedSheetsInParent();
-    } else if (sSharedMemory) {
-      // Use the shared memory handle that was given to us by a SetSharedMemory
-      // call under ContentChild::InitXPCOM.
-      MOZ_ASSERT(sSharedMemory->memory(),
+    } else if (!sSharedMemory.IsEmpty()) {
+      // Use the shared memory that was given to us by a SetSharedMemory call
+      // under ContentChild::InitXPCOM.
+      MOZ_ASSERT(sSharedMemory.data(),
                  "GlobalStyleSheetCache::SetSharedMemory should have mapped "
                  "the shared memory");
     }
@@ -281,25 +279,30 @@ GlobalStyleSheetCache::GlobalStyleSheetCache() {
   // In the parent process, this means we'll just leave our eagerly loaded
   // non-shared sheets in the mFooSheet fields.  In a content process, we'll
   // lazily load our own copies of the sheets later.
-  if (sSharedMemory) {
-    if (auto header = static_cast<Header*>(sSharedMemory->memory())) {
+  if (!sSharedMemory.IsEmpty()) {
+    if (const auto* header =
+            reinterpret_cast<const Header*>(sSharedMemory.data())) {
       MOZ_RELEASE_ASSERT(header->mMagic == Header::kMagic);
 
-#define STYLE_SHEET(identifier_, url_, shared_)                    \
-  if (shared_) {                                                   \
-    LoadSheetFromSharedMemory(url_, &m##identifier_##Sheet,        \
-                              eAgentSheetFeatures, header,         \
-                              UserAgentStyleSheetID::identifier_); \
-  }
-#include "mozilla/UserAgentStyleSheetList.h"
-#undef STYLE_SHEET
+      for (auto kind : MakeEnumeratedRange(BuiltInStyleSheet::Count)) {
+        const auto& info = kBuiltInSheetInfo[size_t(kind)];
+        if (info.mFlags & BuiltInStyleSheetFlags::NotShared) {
+          continue;
+        }
+        const auto parsingMode = (info.mFlags & BuiltInStyleSheetFlags::UA)
+                                     ? eAgentSheetFeatures
+                                     : eAuthorSheetFeatures;
+        LoadSheetFromSharedMemory(info.mURL, &mBuiltIns[kind], parsingMode,
+                                  header, kind);
+      }
     }
   }
 }
 
 void GlobalStyleSheetCache::LoadSheetFromSharedMemory(
-    const char* aURL, RefPtr<StyleSheet>* aSheet, SheetParsingMode aParsingMode,
-    Header* aHeader, UserAgentStyleSheetID aSheetID) {
+    const nsACString& aURL, RefPtr<StyleSheet>* aSheet,
+    SheetParsingMode aParsingMode, const Header* aHeader,
+    BuiltInStyleSheet aSheetID) {
   auto i = size_t(aSheetID);
 
   auto sheet =
@@ -310,12 +313,11 @@ void GlobalStyleSheetCache::LoadSheetFromSharedMemory(
 
   sheet->SetPrincipal(nsContentUtils::GetSystemPrincipal());
   sheet->SetURIs(uri, uri, uri);
-  sheet->SetSharedContents(aHeader->mSheets[i]);
-  sheet->SetComplete();
-
   nsCOMPtr<nsIReferrerInfo> referrerInfo =
       dom::ReferrerInfo::CreateForExternalCSSResources(sheet);
   sheet->SetReferrerInfo(referrerInfo);
+  sheet->SetSharedContents(aHeader->mSheets[i]);
+  sheet->SetComplete();
   URLExtraData::sShared[i] = sheet->URLData();
 
   *aSheet = std::move(sheet);
@@ -323,10 +325,10 @@ void GlobalStyleSheetCache::LoadSheetFromSharedMemory(
 
 void GlobalStyleSheetCache::InitSharedSheetsInParent() {
   MOZ_ASSERT(XRE_IsParentProcess());
-  MOZ_RELEASE_ASSERT(!sSharedMemory);
+  MOZ_RELEASE_ASSERT(sSharedMemory.IsEmpty());
 
-  auto shm = MakeUnique<base::SharedMemory>();
-  if (NS_WARN_IF(!shm->CreateFreezeable(kSharedMemorySize))) {
+  auto handle = ipc::shared_memory::CreateFreezable(kSharedMemorySize);
+  if (NS_WARN_IF(!handle)) {
     return;
   }
 
@@ -358,31 +360,33 @@ void GlobalStyleSheetCache::InitSharedSheetsInParent() {
 #endif
 
   void* address = nullptr;
-  if (void* p = base::SharedMemory::FindFreeAddressSpace(2 * kOffset)) {
+  if (void* p = ipc::shared_memory::FindFreeAddressSpace(2 * kOffset)) {
     address = reinterpret_cast<void*>(uintptr_t(p) + kOffset);
   }
 
-  if (!shm->Map(kSharedMemorySize, address)) {
+  auto mapping = std::move(handle).Map(address);
+  if (!mapping) {
     // Failed to map at the address we computed for some reason.  Fall back
     // to just allocating at a location of the OS's choosing, and hope that
     // it works in the content process.
-    if (NS_WARN_IF(!shm->Map(kSharedMemorySize))) {
+    auto handle = std::move(mapping).Unmap();
+    mapping = std::move(handle).Map();
+    if (NS_WARN_IF(!mapping)) {
       return;
     }
   }
-  address = shm->memory();
+  address = mapping.Address();
 
-  auto header = static_cast<Header*>(address);
+  auto* header = static_cast<Header*>(address);
   header->mMagic = Header::kMagic;
 #ifdef DEBUG
-  for (auto ptr : header->mSheets) {
+  for (const auto* ptr : header->mSheets) {
     MOZ_RELEASE_ASSERT(!ptr, "expected shared memory to have been zeroed");
   }
 #endif
 
-  UniquePtr<RawServoSharedMemoryBuilder> builder(
-      Servo_SharedMemoryBuilder_Create(
-          header->mBuffer, kSharedMemorySize - offsetof(Header, mBuffer)));
+  UniquePtr<StyleSharedMemoryBuilder> builder(Servo_SharedMemoryBuilder_Create(
+      header->mBuffer, kSharedMemorySize - offsetof(Header, mBuffer)));
 
   nsCString message;
 
@@ -391,32 +395,29 @@ void GlobalStyleSheetCache::InitSharedSheetsInParent() {
   // Normally calling ToShared on UA sheets should not fail.  It happens
   // in practice in odd cases that seem like corrupted installations; see bug
   // 1621773.  On failure, return early and fall back to non-shared sheets.
-#define STYLE_SHEET(identifier_, url_, shared_)                      \
-  if (shared_) {                                                     \
-    StyleSheet* sheet = identifier_##Sheet();                        \
-    size_t i = size_t(UserAgentStyleSheetID::identifier_);           \
-    URLExtraData::sShared[i] = sheet->URLData();                     \
-    header->mSheets[i] = sheet->ToShared(builder.get(), message);    \
-    if (!header->mSheets[i]) {                                       \
-      CrashReporter::AppendAppNotesToCrashReport("\n"_ns + message); \
-      return;                                                        \
-    }                                                                \
+  for (auto kind : MakeEnumeratedRange(BuiltInStyleSheet::Count)) {
+    auto i = size_t(kind);
+    const auto& info = kBuiltInSheetInfo[i];
+    if (info.mFlags & BuiltInStyleSheetFlags::NotShared) {
+      continue;
+    }
+    StyleSheet* sheet = BuiltInSheet(kind);
+    URLExtraData::sShared[i] = sheet->URLData();
+    header->mSheets[i] = sheet->ToShared(builder.get(), message);
+    if (!header->mSheets[i]) {
+      CrashReporter::AppendAppNotesToCrashReport("\n"_ns + message);
+      return;
+    }
   }
-#include "mozilla/UserAgentStyleSheetList.h"
-#undef STYLE_SHEET
 
   // Finished writing into the shared memory.  Freeze it, so that a process
   // can't confuse other processes by changing the UA style sheet contents.
-  if (NS_WARN_IF(!shm->Freeze())) {
+  auto readOnlyHandle = std::move(mapping).Freeze();
+  if (NS_WARN_IF(!readOnlyHandle)) {
     return;
   }
 
-  // The Freeze() call unmaps the shared memory.  Re-map it again as read only.
-  // If this fails, due to something else being mapped into the same place
-  // between the Freeze() and Map() call, we can just fall back to keeping our
-  // own copy of the UA style sheets in the parent, and still try sending the
-  // shared memory to the content processes.
-  shm->Map(kSharedMemorySize, address);
+  auto roMapping = readOnlyHandle.Map(address);
 
   // Record how must of the shared memory we have used, for memory reporting
   // later.  We round up to the nearest page since the free space at the end
@@ -425,12 +426,13 @@ void GlobalStyleSheetCache::InitSharedSheetsInParent() {
   // TODO(heycam): This won't be true on Windows unless we allow creating the
   // shared memory with SEC_RESERVE so that the pages are reserved but not
   // committed.
-  size_t pageSize = ipc::SharedMemory::SystemPageSize();
+  size_t pageSize = ipc::shared_memory::SystemPageSize();
   sUsedSharedMemory =
       (Servo_SharedMemoryBuilder_GetLength(builder.get()) + pageSize - 1) &
       ~(pageSize - 1);
 
-  sSharedMemory = shm.release();
+  sSharedMemory = std::move(roMapping).Release();
+  sSharedMemoryHandle() = std::move(readOnlyHandle);
 }
 
 GlobalStyleSheetCache::~GlobalStyleSheetCache() {
@@ -470,7 +472,9 @@ void GlobalStyleSheetCache::InitFromProfile() {
   if (appInfo) {
     bool inSafeMode = false;
     appInfo->GetInSafeMode(&inSafeMode);
-    if (inSafeMode) return;
+    if (inSafeMode) {
+      return;
+    }
   }
   nsCOMPtr<nsIFile> contentFile;
   nsCOMPtr<nsIFile> chromeFile;
@@ -482,7 +486,9 @@ void GlobalStyleSheetCache::InitFromProfile() {
   }
 
   contentFile->Clone(getter_AddRefs(chromeFile));
-  if (!chromeFile) return;
+  if (!chromeFile) {
+    return;
+  }
 
   contentFile->Append(u"userContent.css"_ns);
   chromeFile->Append(u"userChrome.css"_ns);
@@ -492,7 +498,7 @@ void GlobalStyleSheetCache::InitFromProfile() {
 }
 
 RefPtr<StyleSheet> GlobalStyleSheetCache::LoadSheetURL(
-    const char* aURL, SheetParsingMode aParsingMode,
+    const nsACString& aURL, SheetParsingMode aParsingMode,
     FailureAction aFailureAction) {
   nsCOMPtr<nsIURI> uri;
   NS_NewURI(getter_AddRefs(uri), aURL);
@@ -539,11 +545,6 @@ RefPtr<StyleSheet> GlobalStyleSheetCache::LoadSheet(
     gCSSLoader = new Loader;
   }
 
-  // Note: The parallel parsing code assume that UA sheets are always loaded
-  // synchronously like they are here, and thus that we'll never attempt
-  // parallel parsing on them. If that ever changes, we'll either need to find a
-  // different way to prohibit parallel parsing for UA sheets, or handle
-  // -moz-bool-pref and various other things in the parallel parsing code.
   auto result = gCSSLoader->LoadSheetSync(aURI, aParsingMode,
                                           css::Loader::UseSystemPrincipal::Yes);
   if (MOZ_UNLIKELY(result.isErr())) {
@@ -557,139 +558,25 @@ RefPtr<StyleSheet> GlobalStyleSheetCache::LoadSheet(
   return result.unwrapOr(nullptr);
 }
 
-/* static */
-void GlobalStyleSheetCache::InvalidatePreferenceSheets() {
-  if (gStyleCache) {
-    gStyleCache->mContentPreferenceSheet = nullptr;
-    gStyleCache->mChromePreferenceSheet = nullptr;
-  }
-}
-
-void GlobalStyleSheetCache::BuildPreferenceSheet(
-    RefPtr<StyleSheet>* aSheet, const PreferenceSheet::Prefs& aPrefs) {
-  *aSheet = new StyleSheet(eAgentSheetFeatures, CORS_NONE, dom::SRIMetadata());
-
-  StyleSheet* sheet = *aSheet;
-
-  nsCOMPtr<nsIURI> uri;
-  NS_NewURI(getter_AddRefs(uri), "about:PreferenceStyleSheet");
-  MOZ_ASSERT(uri, "URI creation shouldn't fail");
-
-  sheet->SetURIs(uri, uri, uri);
-  sheet->SetComplete();
-
-  static const uint32_t kPreallocSize = 1024;
-
-  nsCString sheetText;
-  sheetText.SetCapacity(kPreallocSize);
-
-#define NS_GET_R_G_B(color_) \
-  NS_GET_R(color_), NS_GET_G(color_), NS_GET_B(color_)
-
-  sheetText.AppendLiteral(
-      "@namespace url(http://www.w3.org/1999/xhtml);\n"
-      "@namespace svg url(http://www.w3.org/2000/svg);\n");
-
-  // Rules for link styling.
-  const bool underlineLinks = StaticPrefs::browser_underline_anchors();
-  sheetText.AppendPrintf("*|*:any-link%s { text-decoration: %s; }\n",
-                         underlineLinks ? ":not(svg|a)" : "",
-                         underlineLinks ? "underline" : "none");
-
-  // Rules for focus styling.
-
-  const bool focusRingOnAnything =
-      StaticPrefs::browser_display_focus_ring_on_anything();
-  uint8_t focusRingWidth = StaticPrefs::browser_display_focus_ring_width();
-  uint8_t focusRingStyle = StaticPrefs::browser_display_focus_ring_style();
-
-  if ((focusRingWidth != 1 && focusRingWidth <= 4) || focusRingOnAnything) {
-    if (focusRingWidth != 1) {
-      // If the focus ring width is different from the default, fix buttons
-      // with rings.
-      sheetText.AppendPrintf(
-          "button::-moz-focus-inner, input[type=\"reset\"]::-moz-focus-inner, "
-          "input[type=\"button\"]::-moz-focus-inner, "
-          "input[type=\"submit\"]::-moz-focus-inner { "
-          "border: %dpx %s transparent !important; }\n",
-          focusRingWidth, focusRingStyle == 0 ? "solid" : "dotted");
-
-      sheetText.AppendLiteral(
-          "button:focus::-moz-focus-inner, "
-          "input[type=\"reset\"]:focus::-moz-focus-inner, "
-          "input[type=\"button\"]:focus::-moz-focus-inner, "
-          "input[type=\"submit\"]:focus::-moz-focus-inner { "
-          "border-color: ButtonText !important; }\n");
-    }
-
-    sheetText.AppendPrintf(
-        "%s { outline: %dpx %s !important; }\n",
-        focusRingOnAnything ? ":focus" : "*|*:link:focus, *|*:visited:focus",
-        focusRingWidth,
-        focusRingStyle == 0 ? "solid -moz-mac-focusring" : "dotted WindowText");
-  }
-
-  if (StaticPrefs::browser_display_use_focus_colors()) {
-    const auto& colors = aPrefs.mLightColors;
-    nscolor focusText = colors.mFocusText;
-    nscolor focusBG = colors.mFocusBackground;
-    sheetText.AppendPrintf(
-        "*:focus, *:focus > font { color: #%02x%02x%02x !important; "
-        "background-color: #%02x%02x%02x !important; }\n",
-        NS_GET_R_G_B(focusText), NS_GET_R_G_B(focusBG));
-  }
-
-  NS_ASSERTION(sheetText.Length() <= kPreallocSize,
-               "kPreallocSize should be big enough to build preference style "
-               "sheet without reallocation");
-
-  // NB: The pref sheet never has @import rules, thus no loader.
-  sheet->ParseSheetSync(nullptr, sheetText,
-                        /* aLoadData = */ nullptr,
-                        /* aLineNumber = */ 0);
-
-#undef NS_GET_R_G_B
-}
-
-bool GlobalStyleSheetCache::AffectedByPref(const nsACString& aPref) {
-  const char* prefs[] = {
-      StaticPrefs::GetPrefName_browser_display_show_focus_rings(),
-      StaticPrefs::GetPrefName_browser_display_focus_ring_style(),
-      StaticPrefs::GetPrefName_browser_display_focus_ring_width(),
-      StaticPrefs::GetPrefName_browser_display_focus_ring_on_anything(),
-      StaticPrefs::GetPrefName_browser_display_use_focus_colors(),
-      StaticPrefs::GetPrefName_browser_underline_anchors(),
-  };
-
-  for (const char* pref : prefs) {
-    if (aPref.Equals(pref)) {
-      return true;
-    }
-  }
-
-  return false;
-}
-
 /* static */ void GlobalStyleSheetCache::SetSharedMemory(
-    base::SharedMemoryHandle aHandle, uintptr_t aAddress) {
+    ipc::ReadOnlySharedMemoryHandle aHandle, uintptr_t aAddress) {
   MOZ_ASSERT(!XRE_IsParentProcess());
   MOZ_ASSERT(!gStyleCache, "Too late, GlobalStyleSheetCache already created!");
-  MOZ_ASSERT(!sSharedMemory, "Shouldn't call this more than once");
+  MOZ_ASSERT(sSharedMemory.IsEmpty(), "Shouldn't call this more than once");
 
-  auto shm = MakeUnique<base::SharedMemory>();
-  if (!shm->SetHandle(std::move(aHandle), /* read_only */ true)) {
+  auto mapping = aHandle.Map(reinterpret_cast<void*>(aAddress));
+  if (!mapping) {
     return;
   }
 
-  if (shm->Map(kSharedMemorySize, reinterpret_cast<void*>(aAddress))) {
-    sSharedMemory = shm.release();
-  }
+  sSharedMemory = std::move(mapping).Release();
+  sSharedMemoryHandle() = std::move(aHandle);
 }
 
-base::SharedMemoryHandle GlobalStyleSheetCache::CloneHandle() {
+ipc::ReadOnlySharedMemoryHandle GlobalStyleSheetCache::CloneHandle() {
   MOZ_ASSERT(XRE_IsParentProcess());
-  if (sSharedMemory) {
-    return sSharedMemory->CloneHandle();
+  if (sSharedMemoryHandle().IsValid()) {
+    return sSharedMemoryHandle().Clone();
   }
   return nullptr;
 }
@@ -698,7 +585,7 @@ StaticRefPtr<GlobalStyleSheetCache> GlobalStyleSheetCache::gStyleCache;
 StaticRefPtr<css::Loader> GlobalStyleSheetCache::gCSSLoader;
 StaticRefPtr<nsIURI> GlobalStyleSheetCache::gUserContentSheetURL;
 
-StaticAutoPtr<base::SharedMemory> GlobalStyleSheetCache::sSharedMemory;
+ipc::shared_memory::LeakedReadOnlyMapping GlobalStyleSheetCache::sSharedMemory;
 size_t GlobalStyleSheetCache::sUsedSharedMemory;
 
 }  // namespace mozilla
