@@ -48,7 +48,7 @@ void SharedStyleSheetCache::LoadCompleted(SharedStyleSheetCache* aCache,
     cancelledStatus = NS_BINDING_ABORTED;
     css::SheetLoadData* data = &aData;
     do {
-      if (data->mIsCancelled) {
+      if (data->IsCancelled()) {
         // We only need to mark loads for this loader as cancelled, so as to not
         // fire error events in unrelated documents.
         css::Loader::MarkLoadTreeFailed(*data, data->mLoader);
@@ -64,13 +64,13 @@ void SharedStyleSheetCache::LoadCompleted(SharedStyleSheetCache* aCache,
 
   // Now it's safe to go ahead and notify observers
   for (RefPtr<css::SheetLoadData>& data : datasToNotify) {
-    auto status = data->mIsCancelled ? cancelledStatus : aStatus;
+    auto status = data->IsCancelled() ? cancelledStatus : aStatus;
     data->mLoader->NotifyObservers(*data, status);
   }
 }
 
 void SharedStyleSheetCache::InsertIfNeeded(css::SheetLoadData& aData) {
-  MOZ_ASSERT(aData.mLoader->GetDocument(),
+  MOZ_ASSERT(aData.mLoader->IsDocumentAssociated(),
              "We only cache document-associated sheets");
   LOG("SharedStyleSheetCache::InsertIfNeeded");
   // If we ever start doing this for failed loads, we'll need to adjust the
@@ -90,7 +90,8 @@ void SharedStyleSheetCache::InsertIfNeeded(css::SheetLoadData& aData) {
 
   if (!aData.mURI) {
     LOG("  Inline or constructable style sheet, bailing");
-    // Inline sheet caching happens in Loader::mInlineSheets.
+    // Inline sheet caching happens in Loader::mInlineSheets, where we still
+    // have the input text available.
     // Constructable sheets are not worth caching, they're always unique.
     return;
   }
@@ -109,10 +110,15 @@ void SharedStyleSheetCache::LoadCompletedInternal(
 
   // Go through and deal with the whole linked list.
   auto* data = &aData;
+  auto* networkMetadata = aData.GetNetworkMetadata();
   do {
     MOZ_RELEASE_ASSERT(!data->mSheetCompleteCalled);
     data->mSheetCompleteCalled = true;
 
+    if (!data->mNetworkMetadata) {
+      data->mNetworkMetadata = networkMetadata;
+    }
+
     if (!data->mSheetAlreadyComplete) {
       // If mSheetAlreadyComplete, then the sheet could well be modified between
       // when we posted the async call to SheetComplete and now, since the sheet
@@ -144,7 +150,7 @@ void SharedStyleSheetCache::LoadCompletedInternal(
           // insert them into the tree.
           return false;
         }
-        if (data->mOwningNodeBeforeLoadEvent != data->mSheet->GetOwnerNode()) {
+        if (data->mHadOwnerNode != !!data->mSheet->GetOwnerNode()) {
           // The sheet was already removed from the tree and is no longer the
           // current sheet of the owning node, we can bail.
           return false;
@@ -156,7 +162,6 @@ void SharedStyleSheetCache::LoadCompletedInternal(
         data->mLoader->InsertSheetInTree(*data->mSheet);
       }
       data->mSheet->SetComplete();
-      data->ScheduleLoadEventIfNeeded();
     } else if (data->mSheet->IsApplicable()) {
       if (dom::Document* doc = data->mLoader->GetDocument()) {
         // We post these events for devtools, even though the applicable state
@@ -164,7 +169,6 @@ void SharedStyleSheetCache::LoadCompletedInternal(
         doc->PostStyleSheetApplicableStateChangeEvent(*data->mSheet);
       }
     }
-
     aDatasToNotify.AppendElement(data);
 
     NS_ASSERTION(!data->mParentData || data->mParentData->mPendingChildren != 0,
@@ -188,6 +192,20 @@ void SharedStyleSheetCache::LoadCompletedInternal(
   }
 }
 
+size_t SharedStyleSheetCache::SizeOfIncludingThis(
+    MallocSizeOf aMallocSizeOf) const {
+  size_t n = aMallocSizeOf(this);
+  n += Base::SizeOfExcludingThis(aMallocSizeOf);
+  n += mInlineSheets.ShallowSizeOfExcludingThis(aMallocSizeOf);
+  for (const auto& sheetMap : mInlineSheets) {
+    for (const auto& entry : sheetMap.GetData()) {
+      n += entry.GetKey().SizeOfExcludingThisIfUnshared(aMallocSizeOf);
+      n += entry.GetData()->SizeOfIncludingThis(aMallocSizeOf);
+    }
+  }
+  return n;
+}
+
 NS_IMETHODIMP
 SharedStyleSheetCache::CollectReports(nsIHandleReportCallback* aHandleReport,
                                       nsISupports* aData, bool aAnonymize) {
@@ -200,21 +218,46 @@ SharedStyleSheetCache::CollectReports(nsIHandleReportCallback* aHandleReport,
   return NS_OK;
 }
 
-void SharedStyleSheetCache::Clear(nsIPrincipal* aForPrincipal,
-                                  const nsACString* aBaseDomain) {
+void SharedStyleSheetCache::ClearInProcess(
+    const Maybe<bool>& aChrome, const Maybe<nsCOMPtr<nsIPrincipal>>& aPrincipal,
+    const Maybe<nsCString>& aSchemelessSite,
+    const Maybe<OriginAttributesPattern>& aPattern,
+    const Maybe<nsCString>& aURL) {
+  Base::ClearInProcess(aChrome, aPrincipal, aSchemelessSite, aPattern, aURL);
+  if (!aChrome && !aPrincipal && !aSchemelessSite && !aURL) {
+    mInlineSheets.Clear();
+  }
+  if (aURL) {
+    // Inline sheets don't have a URL.
+    return;
+  }
+
+  for (auto iter = mInlineSheets.Iter(); !iter.Done(); iter.Next()) {
+    if (SharedSubResourceCacheUtils::ShouldClearEntry(
+            nullptr, iter.Key(), iter.Key(), aChrome, aPrincipal,
+            aSchemelessSite, aPattern, aURL)) {
+      iter.Remove();
+    }
+  }
+}
+
+void SharedStyleSheetCache::Clear(
+    const Maybe<bool>& aChrome, const Maybe<nsCOMPtr<nsIPrincipal>>& aPrincipal,
+    const Maybe<nsCString>& aSchemelessSite,
+    const Maybe<OriginAttributesPattern>& aPattern,
+    const Maybe<nsCString>& aURL) {
   using ContentParent = dom::ContentParent;
 
   if (XRE_IsParentProcess()) {
-    auto forPrincipal = aForPrincipal ? Some(RefPtr(aForPrincipal)) : Nothing();
-    auto baseDomain = aBaseDomain ? Some(nsCString(*aBaseDomain)) : Nothing();
-
     for (auto* cp : ContentParent::AllProcesses(ContentParent::eLive)) {
-      Unused << cp->SendClearStyleSheetCache(forPrincipal, baseDomain);
+      Unused << cp->SendClearStyleSheetCache(aChrome, aPrincipal,
+                                             aSchemelessSite, aPattern, aURL);
     }
   }
 
-  if (sInstance) {
-    sInstance->ClearInProcess(aForPrincipal, aBaseDomain);
+  if (sSingleton) {
+    sSingleton->ClearInProcess(aChrome, aPrincipal, aSchemelessSite, aPattern,
+                               aURL);
   }
 }
 
