@@ -12,20 +12,7 @@
 #![crate_name = "to_shmem"]
 #![crate_type = "rlib"]
 
-extern crate cssparser;
-extern crate servo_arc;
-extern crate smallbitvec;
-extern crate smallvec;
-#[cfg(feature = "string_cache")]
-extern crate string_cache;
-
-use servo_arc::{Arc, ThinArc};
-use smallbitvec::{InternalStorage, SmallBitVec};
-use smallvec::{Array, SmallVec};
 use std::alloc::Layout;
-#[cfg(debug_assertions)]
-use std::any::TypeId;
-#[cfg(debug_assertions)]
 use std::collections::HashSet;
 use std::ffi::CString;
 use std::isize;
@@ -34,8 +21,6 @@ use std::mem::{self, ManuallyDrop};
 use std::num::Wrapping;
 use std::ops::Range;
 use std::os::raw::c_char;
-#[cfg(debug_assertions)]
-use std::os::raw::c_void;
 use std::ptr::{self, NonNull};
 use std::slice;
 use std::str;
@@ -59,16 +44,12 @@ pub struct SharedMemoryBuilder {
     /// The current position in the buffer, where the next value will be written
     /// at.
     index: usize,
-    /// Pointers to every sharable value that we store in the shared memory
+    /// Pointers to every shareable value that we store in the shared memory
     /// buffer.  We use this to assert against encountering the same value
     /// twice, e.g. through another Arc reference, so that we don't
     /// inadvertently store duplicate copies of values.
-    #[cfg(debug_assertions)]
-    shared_values: HashSet<*const c_void>,
-    /// Types of values that we may duplicate in the shared memory buffer when
-    /// there are shared references to them, such as in Arcs.
-    #[cfg(debug_assertions)]
-    allowed_duplication_types: HashSet<TypeId>,
+    #[cfg(all(debug_assertions, feature = "servo_arc"))]
+    shared_values: HashSet<*const std::os::raw::c_void>,
 }
 
 /// Amount of padding needed after `size` bytes to ensure that the following
@@ -89,21 +70,11 @@ impl SharedMemoryBuilder {
             buffer,
             capacity,
             index: 0,
-            #[cfg(debug_assertions)]
+            #[cfg(all(debug_assertions, feature = "servo_arc"))]
             shared_values: HashSet::new(),
-            #[cfg(debug_assertions)]
-            allowed_duplication_types: HashSet::new(),
         }
     }
 
-    /// Notes a type as being allowed for duplication when being copied to the
-    /// shared memory buffer, such as Arcs referencing the same value.
-    #[inline]
-    pub fn add_allowed_duplication_type<T: 'static>(&mut self) {
-        #[cfg(debug_assertions)]
-        self.allowed_duplication_types.insert(TypeId::of::<T>());
-    }
-
     /// Returns the number of bytes currently used in the buffer.
     #[inline]
     pub fn len(&self) -> usize {
@@ -229,13 +200,10 @@ impl_trivial_to_shmem!(
     u32,
     u64,
     isize,
-    usize
+    usize,
+    std::num::NonZeroUsize
 );
 
-impl_trivial_to_shmem!(cssparser::RGBA);
-impl_trivial_to_shmem!(cssparser::SourceLocation);
-impl_trivial_to_shmem!(cssparser::TokenSerializationType);
-
 impl<T> ToShmem for PhantomData<T> {
     fn to_shmem(&self, _builder: &mut SharedMemoryBuilder) -> Result<Self> {
         Ok(ManuallyDrop::new(*self))
@@ -251,14 +219,6 @@ impl<T: ToShmem> ToShmem for Range<T> {
     }
 }
 
-impl ToShmem for cssparser::UnicodeRange {
-    fn to_shmem(&self, _builder: &mut SharedMemoryBuilder) -> Result<Self> {
-        Ok(ManuallyDrop::new(cssparser::UnicodeRange {
-            start: self.start,
-            end: self.end,
-        }))
-    }
-}
 
 impl<T: ToShmem, U: ToShmem> ToShmem for (T, U) {
     fn to_shmem(&self, builder: &mut SharedMemoryBuilder) -> Result<Self> {
@@ -400,24 +360,18 @@ impl<T: ToShmem> ToShmem for Vec<T> {
     }
 }
 
-impl<T: ToShmem, A: Array<Item = T>> ToShmem for SmallVec<A> {
-    fn to_shmem(&self, builder: &mut SharedMemoryBuilder) -> Result<Self> {
-        let dest_vec = unsafe {
-            if self.spilled() {
-                // Place the items in a separate allocation in the shared memory
-                // buffer.
-                let dest = to_shmem_slice(self.iter(), builder)? as *mut T;
-                SmallVec::from_raw_parts(dest, self.len(), self.len())
-            } else {
-                // Place the items inline.
-                let mut s = SmallVec::new();
-                to_shmem_slice_ptr(self.iter(), s.as_mut_ptr(), builder)?;
-                s.set_len(self.len());
-                s
-            }
-        };
-
-        Ok(ManuallyDrop::new(dest_vec))
+impl<T: ToShmem, S> ToShmem for HashSet<T, S>
+where
+    Self: Default,
+{
+    fn to_shmem(&self, _builder: &mut SharedMemoryBuilder) -> Result<Self> {
+        if !self.is_empty() {
+            return Err(format!(
+                "ToShmem failed for HashSet: We only support empty sets \
+                 (we don't expect custom properties in UA sheets, they're observable by content)",
+            ));
+        }
+        Ok(ManuallyDrop::new(Self::default()))
     }
 }
 
@@ -432,23 +386,58 @@ impl<T: ToShmem> ToShmem for Option<T> {
     }
 }
 
-impl<T: 'static + ToShmem> ToShmem for Arc<T> {
+#[cfg(feature = "smallvec")]
+impl<T: ToShmem, A: smallvec::Array<Item = T>> ToShmem for smallvec::SmallVec<A> {
+    fn to_shmem(&self, builder: &mut SharedMemoryBuilder) -> Result<Self> {
+        let dest_vec = unsafe {
+            if self.spilled() {
+                // Place the items in a separate allocation in the shared memory
+                // buffer.
+                let dest = to_shmem_slice(self.iter(), builder)? as *mut T;
+                Self::from_raw_parts(dest, self.len(), self.len())
+            } else {
+                // Place the items inline.
+                let mut s = Self::new();
+                to_shmem_slice_ptr(self.iter(), s.as_mut_ptr(), builder)?;
+                s.set_len(self.len());
+                s
+            }
+        };
+
+        Ok(ManuallyDrop::new(dest_vec))
+    }
+}
+
+#[cfg(feature = "servo_arc")]
+impl<A: 'static, B: 'static> ToShmem for servo_arc::ArcUnion<A, B>
+where
+    servo_arc::Arc<A>: ToShmem,
+    servo_arc::Arc<B>: ToShmem,
+{
+    fn to_shmem(&self, builder: &mut SharedMemoryBuilder) -> Result<Self> {
+        use servo_arc::ArcUnionBorrow;
+
+        Ok(ManuallyDrop::new(match self.borrow() {
+            ArcUnionBorrow::First(first) => Self::from_first(ManuallyDrop::into_inner(
+                first.with_arc(|a| a.to_shmem(builder))?,
+            )),
+            ArcUnionBorrow::Second(second) => Self::from_second(ManuallyDrop::into_inner(
+                second.with_arc(|a| a.to_shmem(builder))?,
+            )),
+        }))
+    }
+}
+#[cfg(feature = "servo_arc")]
+impl<T: ToShmem> ToShmem for servo_arc::Arc<T> {
     fn to_shmem(&self, builder: &mut SharedMemoryBuilder) -> Result<Self> {
         // Assert that we don't encounter any shared references to values we
-        // don't expect.  Those we expect are those noted by calling
-        // add_allowed_duplication_type, and should be types where we're fine
-        // with duplicating any shared references in the shared memory buffer.
-        //
-        // Unfortunately there's no good way to print out the exact type of T
-        // in the assertion message.
+        // don't expect.
         #[cfg(debug_assertions)]
         assert!(
-            !builder.shared_values.contains(&self.heap_ptr()) ||
-                builder
-                    .allowed_duplication_types
-                    .contains(&TypeId::of::<T>()),
-            "ToShmem failed for Arc<T>: encountered a value of type T with multiple references \
-             and which has not been explicitly allowed with an add_allowed_duplication_type call",
+            !builder.shared_values.contains(&self.heap_ptr()),
+            "ToShmem failed for Arc<{}>: encountered a value with multiple \
+            references.",
+            std::any::type_name::<T>()
         );
 
         // Make a clone of the Arc-owned value with all of its heap allocations
@@ -458,7 +447,7 @@ impl<T: 'static + ToShmem> ToShmem for Arc<T> {
         // Create a new Arc with the shared value and have it place its
         // ArcInner in the shared memory buffer.
         unsafe {
-            let static_arc = Arc::new_static(
+            let static_arc = Self::new_static(
                 |layout| builder.alloc(layout),
                 ManuallyDrop::into_inner(value),
             );
@@ -470,8 +459,8 @@ impl<T: 'static + ToShmem> ToShmem for Arc<T> {
         }
     }
 }
-
-impl<H: 'static + ToShmem, T: 'static + ToShmem> ToShmem for ThinArc<H, T> {
+#[cfg(feature = "servo_arc")]
+impl<H: ToShmem, T: ToShmem> ToShmem for servo_arc::Arc<servo_arc::HeaderSlice<H, T>> {
     fn to_shmem(&self, builder: &mut SharedMemoryBuilder) -> Result<Self> {
         // We don't currently have any shared ThinArc values in stylesheets,
         // so don't support them for now.
@@ -484,31 +473,92 @@ impl<H: 'static + ToShmem, T: 'static + ToShmem> ToShmem for ThinArc<H, T> {
 
         // Make a clone of the Arc-owned header and slice values with all of
         // their heap allocations placed in the shared memory buffer.
-        let header = self.header.header.to_shmem(builder)?;
-        let mut values = Vec::with_capacity(self.slice.len());
-        for v in self.slice.iter() {
+        let header = self.header.to_shmem(builder)?;
+        let mut values = Vec::with_capacity(self.len());
+        for v in self.slice().iter() {
             values.push(v.to_shmem(builder)?);
         }
 
         // Create a new ThinArc with the shared value and have it place
         // its ArcInner in the shared memory buffer.
+        let len = values.len();
+        let static_arc = Self::from_header_and_iter_alloc(
+            |layout| builder.alloc(layout),
+            ManuallyDrop::into_inner(header),
+            values.into_iter().map(ManuallyDrop::into_inner),
+            len,
+            /* is_static = */ true,
+        );
+
+        #[cfg(debug_assertions)]
+        builder.shared_values.insert(self.heap_ptr());
+
+        Ok(ManuallyDrop::new(static_arc))
+    }
+}
+
+#[cfg(feature = "thin-vec")]
+impl<T: ToShmem> ToShmem for thin_vec::ThinVec<T> {
+    fn to_shmem(&self, builder: &mut SharedMemoryBuilder) -> Result<Self> {
+        assert_eq!(mem::size_of::<Self>(), mem::size_of::<*const ()>());
+
+        // NOTE: We need to do the work of allocating the header in shared memory even if the
+        // length is zero, because an empty ThinVec, even though it doesn't allocate, references
+        // static memory which will not be mapped to other processes, see bug 1841011.
+        let len = self.len();
+
+        // nsTArrayHeader size.
+        // FIXME: Would be nice not to hard-code this, but in practice thin-vec crate also relies
+        // on this.
+        let header_size = 2 * mem::size_of::<u32>();
+        let header_align = mem::size_of::<u32>();
+
+        let item_size = mem::size_of::<T>();
+        let item_align = mem::align_of::<T>();
+
+        // We don't need to support underalignment for now, this could be supported if needed.
+        assert!(item_align >= header_align);
+
+        // This is explicitly unsupported by ThinVec, see:
+        // https://searchfox.org/mozilla-central/rev/ad732108b073742d7324f998c085f459674a6846/third_party/rust/thin-vec/src/lib.rs#375-386
+        assert!(item_align <= header_size);
+        let header_padding = 0;
+
+        let layout = Layout::from_size_align(
+            header_size + header_padding + padded_size(item_size, item_align) * len,
+            item_align,
+        )
+        .unwrap();
+
+        let shmem_header_ptr = builder.alloc::<u8>(layout);
+        let shmem_data_ptr = unsafe { shmem_header_ptr.add(header_size + header_padding) };
+
+        let data_ptr = self.as_ptr() as *const T as *const u8;
+        let header_ptr = unsafe { data_ptr.sub(header_size + header_padding) };
+
         unsafe {
-            let static_arc = ThinArc::static_from_header_and_iter(
-                |layout| builder.alloc(layout),
-                ManuallyDrop::into_inner(header),
-                values.into_iter().map(ManuallyDrop::into_inner),
-            );
+            // Copy the header. Note this might copy a wrong capacity, but it doesn't matter,
+            // because shared memory ptrs are immutable anyways, and we can't relocate.
+            ptr::copy(header_ptr, shmem_header_ptr, header_size);
+            // ToShmem + copy the contents into the shared buffer.
+            to_shmem_slice_ptr(self.iter(), shmem_data_ptr as *mut T, builder)?;
+            // Return the new ThinVec, which is just a pointer to the shared memory buffer.
+            let shmem_thinvec: Self = mem::transmute(shmem_header_ptr);
 
-            #[cfg(debug_assertions)]
-            builder.shared_values.insert(self.heap_ptr());
+            // Sanity-check that the ptr and length match.
+            debug_assert_eq!(shmem_thinvec.as_ptr(), shmem_data_ptr as *const T);
+            debug_assert_eq!(shmem_thinvec.len(), len);
 
-            Ok(ManuallyDrop::new(static_arc))
+            Ok(ManuallyDrop::new(shmem_thinvec))
         }
     }
 }
 
-impl ToShmem for SmallBitVec {
+#[cfg(feature = "smallbitvec")]
+impl ToShmem for smallbitvec::SmallBitVec {
     fn to_shmem(&self, builder: &mut SharedMemoryBuilder) -> Result<Self> {
+        use smallbitvec::InternalStorage;
+
         let storage = match self.clone().into_storage() {
             InternalStorage::Spilled(vs) => {
                 // Reserve space for the boxed slice values.
@@ -528,7 +578,7 @@ impl ToShmem for SmallBitVec {
             InternalStorage::Inline(x) => InternalStorage::Inline(x),
         };
         Ok(ManuallyDrop::new(unsafe {
-            SmallBitVec::from_storage(storage)
+            Self::from_storage(storage)
         }))
     }
 }
@@ -546,3 +596,19 @@ impl<Static: string_cache::StaticAtomSet> ToShmem for string_cache::Atom<Static>
         )
     }
 }
+
+#[cfg(feature = "cssparser")]
+impl_trivial_to_shmem!(
+    cssparser::SourceLocation,
+    cssparser::SourcePosition,
+    cssparser::TokenSerializationType
+);
+#[cfg(feature = "cssparser")]
+impl ToShmem for cssparser::UnicodeRange {
+    fn to_shmem(&self, _builder: &mut SharedMemoryBuilder) -> Result<Self> {
+        Ok(ManuallyDrop::new(Self {
+            start: self.start,
+            end: self.end,
+        }))
+    }
+}
